from furl import furl

from django.contrib import messages
from django.http import HttpResponse
from django.shortcuts import redirect
from django.urls import reverse

from .clone import clone_object
from .helpers import get_model_obj, redirect_to_referer
from .models import DynamicFilterExpr


def dynfilters_add(request, model_name):
    try:
        model_obj = get_model_obj(model_name)
    except:
        messages.error(request, 'This type of model is unknown.')
        return redirect_to_referer(request)

    expr = DynamicFilterExpr.objects.create(
        model=model_name,
        user=request.user,
    )

    url = reverse('admin:dynfilters_dynamicfilterexpr_change', args=(expr.pk,))

    return redirect(f'{url}?next={request.GET.get("next")}')

def dynfilters_share(request, id):
    try:
        expr = DynamicFilterExpr.objects.get(pk=id)
    except DynamicFilterExpr.DoesNotExist:
        messages.error(request, 'This filter does not exist.')
        return redirect(reverse('admin:dynfilters_dynamicfilterexpr_changelist'))

    clone = clone_object(expr)
    clone.user = request.user
    clone.save()

    return redirect(reverse('admin:dynfilters_dynamicfilterexpr_change', args=(clone.pk,)))

def dynfilters_delete(request, id):
    try:
        expr = DynamicFilterExpr.objects.get(pk=id)
    except DynamicFilterExpr.DoesNotExist:
        messages.error(request, 'This filter does not exist.')
        return redirect_to_referer(request)

    expr.delete()

    return redirect_to_referer(request)



# Generated by Django 4.0.5 on 2022-06-07 19:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynfilters', '0003_alter_dynamicfilterexpr_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dynamicfilterexpr',
            name='is_global',
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name='dynamicfilterexpr',
            name='model',
            field=models.CharField(db_index=True, max_length=64),
        ),
    ]



from django.contrib import admin, messages
from django.contrib.admin import AdminSite
from django.contrib.auth.models import User
from django.db.models import Q
from django.urls import reverse

from .helpers import (
    get_model_admin,
    get_qualified_model_names,
    get_dynfilters_fields,
    get_dynfilters_select_related,
    get_dynfilters_prefetch_related,
)
from .models import DynamicFilterExpr
from .utils import flatten


class DynamicFilter(admin.SimpleListFilter):
    title = 'dynamic filter'
    parameter_name = "filter"
    template = 'dynfilters/dynamic_filter.html'

    email_subject = 'Sharing filter'
    email_text = 'Hi,\nI would like to share this filter with you:\n\n'

    def __init__(self, request, params, model, model_admin):
        self.request = request
        self.referer = request.build_absolute_uri()
        self.model_name = get_qualified_model_names(model._meta)[0]

        return super().__init__(request, params, model, model_admin)

    def has_output(self):
        # We want the filter to be displayed even when there are no reports,
        # so that the '+' is always present.
        return True

    def choices(self, changelist):
        yield {
            "selected": self.value() is None,
            "query_string": changelist.get_query_string(remove=[self.parameter_name]),
            "display": "All",
        }
        for lookup, title in self.lookup_choices:
            yield {
                "selected": self.value() == str(lookup),
                "query_string": changelist.get_query_string(
                    {self.parameter_name: lookup}
                ),
                "display": title,
                "lookup": lookup,
                "email_body": self.request.build_absolute_uri(
                    reverse('dynfilters_share', args=(lookup,))
                ),
            }

    def lookups(self, request, model_admin):
        model_names = get_qualified_model_names(model_admin.opts)
        
        return [
            (o.pk, o.name)
            for o in (
                DynamicFilterExpr
                    .objects
                    .filter(model__in=model_names)
                    .filter(Q(user=request.user) | Q(is_global=True))
                    .order_by('name')
                )
        ]

    def queryset(self, request, queryset):
        if self.value() is not None:
            try:
                obj = DynamicFilterExpr.objects.get(pk=self.value())
            except DynamicFilterExpr.DoesNotExist:
                return queryset # filter no longer exists

            model_admin = get_model_admin(obj)

            elementary_fields = flatten([
                f.split('__') 
                for f, display in get_dynfilters_fields(model_admin)
                if f != '-'
            ])

            queryset = queryset.select_related(*[
                f 
                for f in get_dynfilters_select_related(model_admin)
                if f in elementary_fields
            ])

            queryset = queryset.prefetch_related(*[
                f 
                for f in get_dynfilters_prefetch_related(model_admin)
                if f in elementary_fields
            ])

            try:
                return queryset.filter(obj.as_q())
            except:
                messages.error(request, 'The selected filter is buggy and needs to be corrected.')
                return queryset



# Generated by Django 4.0.5 on 2022-06-02 17:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynfilters', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dynamicfilterexpr',
            name='model',
            field=models.CharField(default='test', max_length=64),
            preserve_default=False,
        ),
    ]



from furl import furl

from django import forms
from django.apps import apps
from django.contrib import admin
from django.shortcuts import redirect
from django.utils.html import format_html, format_html_join

from adminsortable2.admin import SortableAdminBase, SortableInlineAdminMixin

from .models import (
    DynamicFilterExpr,
    DynamicFilterTerm,
    DynamicFilterColumn,
    DynamicFilterColumnSortOrder,
)

from .forms import (
    DynamicFilterExprForm,
    DynamicFilterTermInlineForm,
    DynamicFilterTermInlineFormSet,
)

from .helpers import (
    get_model_admin,
    get_model_choices,
    get_dynfilters_fields,
)


class DynamicFilterInline(admin.TabularInline):
    extra = 0

    def formfield_for_dbfield(self, db_field, **kwargs):
        obj = kwargs['request'].parent_object

        if db_field.name == 'field' and obj:
            model_admin = get_model_admin(obj)
            kwargs['widget'] = forms.Select(choices=get_dynfilters_fields(model_admin))

        return super().formfield_for_dbfield(db_field, **kwargs)

class DynamicFilterTermInline(SortableInlineAdminMixin, DynamicFilterInline):
    model = DynamicFilterTerm
    form = DynamicFilterTermInlineForm
    formset = DynamicFilterTermInlineFormSet
    verbose_name = 'Search Criteria'
    verbose_name_plural = 'Search Criterias'


class DynamicFilterColumnInline(SortableInlineAdminMixin, DynamicFilterInline):
    model = DynamicFilterColumn
    verbose_name = 'Display Column'
    verbose_name_plural = 'Display Columns'


class DynamicFilterColumnSortOrderInline(SortableInlineAdminMixin, DynamicFilterInline):
    model = DynamicFilterColumnSortOrder
    verbose_name = 'Sort Order'
    verbose_name_plural = 'Sort Orders'


def get_next_url(request):
    f = furl(request.META["HTTP_REFERER"])
    return f.args.get('next')

@admin.register(DynamicFilterExpr)
class DynamicFilterExprAdmin(SortableAdminBase, admin.ModelAdmin):
    form = DynamicFilterExprForm
    inlines = [DynamicFilterTermInline]#, DynamicFilterColumnInline, DynamicFilterColumnSortOrderInline]

    list_per_page = 50
    list_display = ('_name', 'model', '_creator')

    def get_form(self, request, obj=None, **kwargs):
        request.parent_object = obj
        return super().get_form(request, obj, **kwargs)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'model':
            kwargs['widget'] = forms.Select(choices=get_model_choices())

        if db_field.name == 'user':
            db_field.default = kwargs['request'].user

        return super().formfield_for_dbfield(db_field,**kwargs)

    def response_add(self, request, obj, post_url_continue=None):
        url = get_next_url(request)
        return (
            redirect(url)
            if url else
            super().response_add(request, obj, post_url_continue)
        )

    def response_change(self, request, obj):
        url = get_next_url(request)
        return (
            redirect(url)
            if url else
            super().response_change(request, obj)
        )

    # def response_delete(self, request, obj_display, obj_id):
    #     url = get_next_url(request)
    #     return (
    #         redirect(url)
    #         if url else
    #         super().response_delete(request, obj_display, obj_id)
    #     )

    def _creator(self, obj):
        return obj.user

    def _name(self, obj):
        return format_html(f'<a href="/admin/dynfilters/dynamicfilterexpr/{obj.id}/change/">{obj.name}</a>')



# Generated by Django 4.0.5 on 2022-06-02 08:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DynamicFilterExpr',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=128, null=True)),
                ('model', models.CharField(blank=True, default='Patent', max_length=16, null=True)),
                ('is_global', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Filter',
                'verbose_name_plural': 'Filters',
            },
        ),
        migrations.CreateModel(
            name='DynamicFilterTerm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('op', models.CharField(choices=[('-', '-'), ('!', 'NOT'), ('&', 'AND'), ('|', 'OR'), ('(', '('), (')', ')')], default='-', max_length=1)),
                ('field', models.CharField(blank=True, max_length=64, null=True)),
                ('lookup', models.CharField(choices=[('-', '---------'), ('=', 'Equals'), ('icontains', 'Contains'), ('istartswith', 'Starts with'), ('iendswith', 'Ends with'), ('in', 'One of'), ('-', '---------'), ('range', 'Date Range'), ('year', 'Date Year'), ('month', 'Date Month'), ('day', 'Date Day'), ('-', '---------'), ('isnull', 'Is NULL'), ('isnotnull', 'Is not NULL'), ('istrue', 'Is TRUE'), ('isfalse', 'Is FALSE'), ('-', '---------'), ('lt', 'Less Than'), ('gt', 'Greater Than'), ('lte', 'Less Than or Equal To'), ('gte', 'Greater Than or Equal To')], default='-', max_length=16)),
                ('value', models.CharField(blank=True, max_length=100, null=True)),
                ('bilateral', models.BooleanField(default=False)),
                ('order', models.PositiveSmallIntegerField(blank=True, db_index=True, default=0)),
                ('filter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dynfilters.dynamicfilterexpr')),
            ],
            options={
                'verbose_name': 'Field',
                'verbose_name_plural': 'Fields',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='DynamicFilterColumnSortOrder',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.CharField(blank=True, max_length=64, null=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('filter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dynfilters.dynamicfilterexpr')),
            ],
            options={
                'verbose_name': 'Column sort order',
                'verbose_name_plural': 'Column sort orders',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='DynamicFilterColumn',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.CharField(blank=True, max_length=64, null=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('filter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dynfilters.dynamicfilterexpr')),
            ],
            options={
                'verbose_name': 'Column',
                'verbose_name_plural': 'Columns',
                'ordering': ('order',),
            },
        ),
    ]
